%Input data:
%Adding co-ordinate information for riserbase and wells
RBC = [0 0];
W1= [-16500 -22000]; W2= [-22000 -22000]; W3= [-24500 -17500];
W4= [-29000 -4000]; W5= [-29500 0]; W6= [-26000 4500];
W7= [-23500 18500]; W8= [-22500 22500]; W9= [-18000 24500];
W10= [16500 24500]; W11= [22000 22000]; W12= [21500 17500];
W13= [27000 3000]; W14= [30000 0]; W15= [29500 -4000];
W16= [26500 -20000]; W17= [23000 -23000]; W18= [18500 -22500];

%If additional wells have been added, they should also be added to the 
%below cell array
W={W1 W2 W3 W4 W5 W6 W7 W8 W9 W10 W11 W12 W13 W14 W15 W16 W17 W18};
Wn=numel(W);
     
%Adding minimum allowable radius of curvature and number of well in each
%loop. Also, dx and dy values for the created grid system 
Rmin=2000; %Adding minimum radius of curvature
dx=0.469798658*1000; %dx of one grid in meter
dy=0.463576159*1000; %dy of one grid in meter
Wells_in_loop = 6; %It is assumed that each loop contains same number of wells
Num_loops=Wn/Wells_in_loop;   
%End of Input data

%Determining node number of given co-ordinates
x_RBC= round(abs(x(1,1)-RBC(1,1))/dx+1);
y_RBC= round(abs(y(1,1)-RBC(1,2))/dy+1);
n_RBC= round((y_RBC-1)*n+x_RBC);
x_W=cell(1,Wn);
y_W=cell(1,Wn);
n_W=cell(1,Wn);
for i=1:Wn
x_W{i}= round(abs(x(1,1)-W{i}(1,1))/dx+1);
y_W{i}= round(abs(y(1,1)-W{i}(1,2))/dy+1);
n_W{i}= round((y_W{i}-1)*n+x_W{i});
end

%Dividing wells into several loops (Assuming each loop contains same number
%of wells).
Points_in_loops=cell(1,Num_loops);
for i=1:Num_loops
    Points_in_loops{i}=zeros(1,Wells_in_loop+1);
    Points_in_loops{i}(1)=n_RBC;
    for j=1:Wells_in_loop
        Points_in_loops{i}(j+1)=n_W{j+(i-1)*Wells_in_loop};
    end
    Points_in_loops{i}=[Points_in_loops{i}, n_RBC];
end

% Calculating shortest unsmoothed paths between wells and riserbase 
% by using Dijkstra's algorithm. Also, plotting unsmoothed paths.
G=R;
length_unsmoothed=0;
L_unsmooth=cell(1,(Wells_in_loop+1)*Num_loops);
L_unsmoothed=zeros(1,Num_loops);
UnsmoothPaths=cell(1,(Wells_in_loop+1)*Num_loops);
UnsmoothedPaths=cell(1,Num_loops);
for i=1:Num_loops
    for j=1:Wells_in_loop+1;
    caption1 = sprintf('%d',Points_in_loops{i}(j));
    ni_1 = findnode(G,caption1);
    caption2 = sprintf('%d',Points_in_loops{i}(j+1));
    ni_2 = findnode(G,caption2);
    [P,L_unsmooth{j+(i-1)*(Wells_in_loop+1)}] = shortestpath(G,ni_1,ni_2); %available MATLAB function for the Dijkstra's algorithm
    UnsmoothPaths{j+(i-1)*(Wells_in_loop+1)}(:,1)=G.Nodes{P(1,:),1};
    UnsmoothPaths{j+(i-1)*(Wells_in_loop+1)}(:,2)=G.Nodes{P(1,:),2};
    UnsmoothPaths{j+(i-1)*(Wells_in_loop+1)}(:,3)=G.Nodes{P(1,:),3};
    L_unsmoothed(i)=L_unsmoothed(i)+L_unsmooth{j+(i-1)*(Wells_in_loop+1)};
    a=numel(P);
    figure(2)
    plot3(G.Nodes{P(1,:),1},G.Nodes{P(1,:),2},G.Nodes{P(1,:),3}, 'color', 'b', 'linewidth', 2)
    hold on
    if j<Wells_in_loop+1
    plot3(G.Nodes{P(1,a),1},G.Nodes{P(1,a),2},G.Nodes{P(1,a),3},'b.', 'MarkerSize',  20)
    caption3 = sprintf('W%d', j+(i-1)*Wells_in_loop);
    text(G.Nodes{P(1,a),1},G.Nodes{P(1,a),2},G.Nodes{P(1,a),3},caption3, 'fontsize', 18)
    end
    length_unsmoothed=length_unsmoothed+L_unsmooth{i};
    P(a)=[];
    P(a-1)=[];
    P(1)=[];
    P(2)=[];
    G = rmnode(G,P(:));
    end
end
caption2 = sprintf('%d',n_RBC);
ni_2 = findnode(G,caption2);
plot3(G.Nodes{ni_2,1},G.Nodes{ni_2,2},G.Nodes{ni_2,3},'b.', 'MarkerSize',  20)
text(G.Nodes{ni_2,1},G.Nodes{ni_2,2},G.Nodes{ni_2,3},'RB', 'fontsize', 18)
zlim([-180 180]);   %This figure settings have been made for the used data
xlim([-35000 35000]); %for other data these can be changed 
ylim([-35000 35000]);
xlabel('x');
ylabel('y');
zlabel('z');
pbaspect([1 1 1]);
grid off
hold off

%Smoothing lines to prevent slipping of flowlines while installation
a=numel(UnsmoothPaths);
Tent=12500; %This value can be kept same with Rmin, if smaller grids are used
for i=1:a
    SmoothPaths{i}=pathsmoothing(UnsmoothPaths{i},Tent);                    
end

for i=1:Num_loops
    for j=1:Wells_in_loop+1
       if j>1 
          SmoothPaths{j+(i-1)*(Wells_in_loop+1)}(1,:)=[];
          UnsmoothedPaths{i}=[UnsmoothedPaths{i};SmoothPaths{j+(i-1)*(Wells_in_loop+1)}];
       else
    UnsmoothedPaths{i}=[UnsmoothedPaths{i};SmoothPaths{j+(i-1)*(Wells_in_loop+1)}];
       end
    end
end

for i=1:Num_loops
    SmoothPathsFinal{i}=pathsmoothing(UnsmoothedPaths{i},Rmin);
                        %Written MATLAB function for smoothing process
end

%Plotting Smoothed Lines
for i=1:Num_loops
figure(3)
plot3(SmoothPathsFinal{i}(:,1),SmoothPathsFinal{i}(:,2),SmoothPathsFinal{i}(:,3), 'color', 'b', 'linewidth', 2) ;
zlim([-180 180]);   %This figure settings have been made for the used data
xlim([-35000 35000]) %for other data these can be changed 
ylim([-35000 35000]);
hold on
end

%Plotting Points
for i=1:Wn
    caption1 = sprintf('%d',n_W{i});
    ni_W = findnode(G,caption1);
    plot3(G.Nodes{ni_W,1},G.Nodes{ni_W,2},G.Nodes{ni_W,3},'b.', 'MarkerSize',  20)
    caption3 = sprintf('W%d', i);
    text(G.Nodes{ni_W,1},G.Nodes{ni_W,2},G.Nodes{ni_W,3},caption3, 'fontsize', 18)
end
plot3(G.Nodes{ni_2,1},G.Nodes{ni_2,2},G.Nodes{ni_2,3},'b.', 'MarkerSize',  20)
text(G.Nodes{ni_2,1},G.Nodes{ni_2,2},G.Nodes{ni_2,3},'RB', 'fontsize', 18)
xlabel('x');
ylabel('y');
zlabel('z');
pbaspect([1 1 1]);
grid off
hold off

%Calculating total length of smoothed lines
length_smoothed=0;
L_smoothed=cell(1,Num_loops);
for i=1:Num_loops
    L_smoothed{i}=smoothedpathlength(SmoothPathsFinal{i}(:,1),SmoothPathsFinal{i}(:,2),SmoothPathsFinal{i}(:,3));
                    %Written function for calculating length
    length_smoothed=length_smoothed+L_smoothed{i};
end
X = sprintf('Total length of the flowlines is equal to %d m', length_smoothed);
disp(X);